<?xml version="1.0" encoding="UTF-8"?>
<!--
 - Desc:    This file is part of the eCromedos document preparation system
 - Date:    2006/03/09
 - Author:  Tobias Koch (tkoch@ecromedos.net)
 - License: GNU General Public License, version 2
 - URL:     http://www.ecromedos.net
-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!--
  - Print table of contents.
-->
<xsl:template name="toc.make">

	<!-- depth of toc -->
	<xsl:variable name="tocdepth">
		<xsl:call-template name="util.tocdepth"/>
	</xsl:variable>

	<!-- make title -->
	<xsl:call-template name="toc.head"/>
	<!-- adjust pagestyle -->
	<xsl:text>\pagestyle{scrheadings}&#x0a;</xsl:text>
	<!-- table of contents -->
	<xsl:if test="$tocdepth > 0">
		<xsl:text>\tableofcontents{}&#x0a;</xsl:text>
	</xsl:if>
	<xsl:if test="/report">
		<xsl:text>\clearpage{}&#x0a;</xsl:text>
	</xsl:if>
</xsl:template>

<!--
  - Generate title page.
-->
<xsl:template name="toc.head">
	<!-- subject is optional -->
	<xsl:if test="head/subject">
		<xsl:text>\subject{</xsl:text>
		<xsl:apply-templates select="head/subject"/>
		<xsl:text>}&#x0a;</xsl:text>
	</xsl:if>
	<!-- title is mandatory -->
	<xsl:text>\title{</xsl:text>
	<xsl:apply-templates select="head/title"/>
	<xsl:text>}&#x0a;</xsl:text>
	<!-- subtitle is optional -->
	<xsl:if test="head/subtitle">
		<xsl:text>\subtitle{</xsl:text>
		<xsl:apply-templates select="head/subtitle"/>
		<xsl:text>}&#x0a;</xsl:text>
	</xsl:if>
	<!-- author is mandatory -->
	<xsl:text>\author{</xsl:text>
	<xsl:for-each select="head/author">
		<xsl:if test="preceding-sibling::author">
			<xsl:text>\and </xsl:text>
		</xsl:if>
		<xsl:apply-templates select="."/>
	</xsl:for-each>
	<xsl:text>}&#x0a;</xsl:text>
	<!-- date is optional -->
	<xsl:if test="head/date">
		<xsl:text>\date{</xsl:text>
		<xsl:apply-templates select="head/date"/>
		<xsl:text>}&#x0a;</xsl:text>
	</xsl:if>
	<!-- publisher is optional -->
	<xsl:if test="head/publisher">
		<xsl:text>\publishers{</xsl:text>
		<xsl:apply-templates select="head/publisher"/>
		<xsl:text>}&#x0a;</xsl:text>
	</xsl:if>
	<!-- dedication is optional -->
	<xsl:if test="head/dedication">
		<xsl:text>\dedication{</xsl:text>
		<xsl:apply-templates select="head/dedication"/>
		<xsl:text>}&#x0a;</xsl:text>
	</xsl:if>
	<xsl:text>\maketitle&#x0a;&#x0a;</xsl:text>
</xsl:template>

</xsl:stylesheet>
